/*
 * This file contains the list of permission scopes supported by extensions and various helper
 * functions for using scopes.
 *
 * The enums and helpers in this file are carefully structured to use TypeScript's exhaustive switch
 * case rule to guide you in adding new resources, actions, and scopes. To add new resources;
 * actions; and scopes, all you should need to do is modify the enums appropriately and fix any
 * resulting type errors.
 *
 * For more info, see: https://github.com/netlify/jigsaw/blob/main/scopes/scopes.go
 *
 * TODO(ndhoule): Generate the ExtensionScope automatically from an OpenAPI schema, from Go source,
 * etc.
 */
/**
 * @internal
 */
export var ExtensionResource;
(function (ExtensionResource) {
    ExtensionResource["Extension"] = "extension";
    ExtensionResource["Env"] = "env";
    ExtensionResource["Site"] = "site";
    ExtensionResource["User"] = "user";
})(ExtensionResource || (ExtensionResource = {}));
/**
 * @internal
 */
export var ExtensionAction;
(function (ExtensionAction) {
    ExtensionAction["Read"] = "read";
    ExtensionAction["Write"] = "write";
    ExtensionAction["Delete"] = "delete";
})(ExtensionAction || (ExtensionAction = {}));
/**
 * An enum containing all extension permission scopes. These scopes are primarily used to restrict
 * interactions with the Netlify API (Bitballoon).
 *
 * @internal
 */
export var ExtensionScope;
(function (ExtensionScope) {
    /* eslint-disable @typescript-eslint/prefer-literal-enum-member */
    ExtensionScope["EnvRead"] = "env:read";
    ExtensionScope["EnvWrite"] = "env:write";
    ExtensionScope["EnvDelete"] = "env:delete";
    ExtensionScope["SiteRead"] = "site:read";
    ExtensionScope["SiteWrite"] = "site:write";
    ExtensionScope["UserRead"] = "user:read";
    ExtensionScope["UserWrite"] = "user:write";
    /**
     * @deprecated This scope only exists for v1 extension support. It's used by v2 extensions in the
     * enable flow, but the permissions it grants are not required to enable a v2 extension.
     */
    ExtensionScope["EnableExtension"] = "enable";
    /* eslint-enable @typescript-eslint/prefer-literal-enum-member */
})(ExtensionScope || (ExtensionScope = {}));
const DEPRECATED_SCOPES = new Set([ExtensionScope.EnableExtension]);
const isScopeDeprecated = (scope) => DEPRECATED_SCOPES.has(scope);
/**
 * listExtensionScopes returns a list of all extension scope values.
 *
 * By default, deprecated scopes are filtered out.
 *
 * @internal
 */
export const listExtensionScopes = ({ includeDeprecated = false, } = {}) => {
    const scopes = Object.values(ExtensionScope);
    return includeDeprecated ? scopes : scopes.filter(isScopeDeprecated);
};
const humanizeExtensionResource = (resource) => {
    switch (resource) {
        case ExtensionResource.Extension:
            return "Extension";
        case ExtensionResource.Env:
            return "Environment variables";
        case ExtensionResource.Site:
            return "Site";
        case ExtensionResource.User:
            return "User";
    }
};
const humanizeExtensionAction = (action) => {
    switch (action) {
        case ExtensionAction.Read:
            return "read";
        case ExtensionAction.Write:
            return "write";
        case ExtensionAction.Delete:
            return "delete";
    }
};
/**
 * humanizeExtensionScope returns an object containing human-readable labels for that scope.
 *
 * @internal
 */
export const humanizeExtensionScope = (scope) => {
    switch (scope) {
        case ExtensionScope.EnableExtension:
        case ExtensionScope.EnvRead:
        case ExtensionScope.EnvWrite:
        case ExtensionScope.EnvDelete:
        case ExtensionScope.SiteRead:
        case ExtensionScope.SiteWrite:
        case ExtensionScope.UserRead:
        case ExtensionScope.UserWrite: {
            const [resource, action] = scope.split(":");
            return {
                action: humanizeExtensionAction(action),
                resource: humanizeExtensionResource(resource),
                value: scope,
            };
        }
    }
};
/**
 * deserializeExtensionScopes transforms a serialized list of scopes into an array of
 * ExtensionScopes. Throws if the list is malformed or contains invalid scopes.
 *
 * @internal
 */
export const deserializeExtensionScopes = (value) => {
    const parts = value.split(",").map((part) => part.trim());
    const invalidScopes = parts.filter((part) => !isValidExtensionScope(part));
    if (invalidScopes.length) {
        throw new Error(`Invalid scope(s): ${invalidScopes.map((invalidScope) => `"${invalidScope}"`).join(", ")}`);
    }
    return parts.filter((part) => isValidExtensionScope(part));
};
/**
 * serializeExtensionScopes serializes a list of extension scopes into a format suitable for
 * consumption by the Jigsaw API.
 *
 * @internal
 */
export const serializeExtensionScopes = (scopes) => scopes.slice().sort().join(",");
const ALL_EXTENSION_SCOPES = new Set(listExtensionScopes({ includeDeprecated: true }));
/**
 * isValidExtensionScope checks if a given value is a valid ExtensionScope.
 *
 * @internal
 */
export const isValidExtensionScope = (value) => typeof value === "string" && ALL_EXTENSION_SCOPES.has(value);
const getExtensionScopeResourceRank = (action) => {
    switch (action) {
        case ExtensionResource.Site:
            return 1;
        case ExtensionResource.Env:
            return 2;
        case ExtensionResource.User:
            return 3;
        default:
            return 1000;
    }
};
/**
 * @internal
 */
export const extensionResourceComparator = (resourceA, resourceB) => {
    const a = getExtensionScopeResourceRank(resourceA);
    const b = getExtensionScopeResourceRank(resourceB);
    return a === b ? 0 : a < b ? -1 : 1;
};
const getExtensionScopeActionRank = (action) => {
    switch (action) {
        case ExtensionAction.Read:
            return 1;
        case ExtensionAction.Write:
            return 2;
        case ExtensionAction.Delete:
            return 3;
        default:
            return 1000;
    }
};
/**
 * @internal
 */
export const extensionActionComparator = (actionA, actionB) => {
    const a = getExtensionScopeActionRank(actionA);
    const b = getExtensionScopeActionRank(actionB);
    return a === b ? 0 : a < b ? -1 : 1;
};
//# sourceMappingURL=scopes.js.map